const path = require('path')
const electron = require('electron')
const app = electron.app
const BrowserWindow = electron.BrowserWindow
const nativeTheme = electron.nativeTheme
const shortcut = require('./shortcut')

const Store = require('electron-store');
const store = new Store();
if (!store.get('iniSetting')) {
  store.set('iniSetting', true);
  store.set('minimize', true);
  store.set('autoStart', false);
  store.set('notice.bubble', true);
  store.set('notice.winShow', false);
  store.set('notice.twinkle', true);
}
if (!store.get('iniSetting_130')) {
  store.set('iniSetting_130', true);
  store.set('winWidth', 950);
  store.set('winHeight', 700);
  store.set('winMaximize', false);
  store.set('winX', null);
  store.set('winY', null);
  store.set('userList', []);
}

const mode = 'product'; //product
let mainWinStatus = "show" //win的状态
let imageMenu = null //传过来的要复制的图片
let isLoginOut = false //是否退出了
let config = {
  edition: '1.5.0',
  workUrl: mode == 'test' ? 'http://127.0.0.1:8081/elogin' : 'https://work.yingyuchat.com/elogin',
  isMac: process.platform == "darwin" ? true : false
}

let win = null, //主窗口
  setWin = null, //设置页面窗口
  nofcWin = null, //用来发送桌面通知的隐藏窗口
  setting = {
    minimize: store.get('minimize'),
    autoStart: store.get('autoStart'),
    'notice.bubble': store.get('notice.bubble'),
    'notice.winShow': store.get('notice.winShow'),
    'notice.twinkle': store.get('notice.twinkle'),
    winWidth: store.get('winWidth'),
    winHeight: store.get('winHeight'),
    winMaximize: store.get('winMaximize'),
    winX: store.get('winX'),
    winY: store.get('winY'),
  };

//系统托盘图标闪烁
let count = 0,
  timer = null,
  appTray = null,
  aSeleIcon = path.join(path.join(__dirname), 'img/osxicon_selected.png'),
  aIcon = path.join(path.join(__dirname), 'img/aicon.png'),
  bIcon = path.join(path.join(__dirname), 'img/bicon.png'),
  ticon = path.join(path.join(__dirname), 'img/ticon.png');

//判断是否为OSX
if (process.platform == "darwin") {
  setTrayImage()
  //当桌面主题更新时
  nativeTheme.on('updated', () => {
    setTrayImage()
    appTray.setImage(aIcon)
  })
}
//苹果设置托盘
function setTrayImage() {
  aIcon = path.join(path.join(__dirname), 'img/osxicon_dark.png')
  // if (nativeTheme.shouldUseDarkColors) {
  //   aIcon = path.join(path.join(__dirname), 'img/osxicon_dark.png')
  // } else {
  //   aIcon = path.join(path.join(__dirname), 'img/osxicon.png')
  // }
}
//准备就绪后执行
function createWindow() {
  app.setAppUserModelId('yingyuchatcomwork')

  // 创建浏览器窗口。
  let windowOptions = {
    width: setting.winWidth,
    height: setting.winHeight,
    // show: false,
    backgroundColor: '#fcfcfc',
    webPreferences: {
      nodeIntegration: true,
      contextIsolation:false,
    }
  }
  if (setting.winX || setting.winX === 0) windowOptions.x = setting.winX
  if (setting.winY || setting.winY === 0) windowOptions.y = setting.winY
  windowOptions.icon = path.join(__dirname, 'img/aicon.png');
  var tmwin=null;
  try{
  	tmwin=new BrowserWindow(windowOptions);
  }catch(e){
  	
  }
  win = tmwin;
  //if (setting.winMaximize) win.maximize();

  // 加载应用的 index.html。
  // win.loadFile('app/html/index.html')
  try{
  win.loadURL(`file://${__dirname}/html/index.html`)
  
  // win.loadURL(`${config.workUrl}?electron=${config.edition}`)

  //页面加载完成后发消息通知
  win.webContents.on('did-finish-load', () => {
    win.webContents.send('contact', { type: 'ini', data: { config } })
  })

  // //网页加载完成后显示窗口
  // win.once('ready-to-show', () => {
  //   win.show()
  // })

  //网页target链接让用户电脑里的默认浏览器打开
  win.webContents.on('new-window', (event, url) => {
    event.preventDefault()
    if (!url) return
    electron.shell.openExternal(url.toString())
    // //收到新消息需要提醒
    // url = url.toString();
    // if(url.substr(0,20) == 'http://notification/'){

    // }else {
    //   const shell = electron.shell
    //   shell.openExternal(url)
    // }
  })

  //隐藏顶部菜单
  win.setMenu(null);

  win.on('show', () => {
    if (timer) clearInterval(timer);
    appTray.setImage(aIcon) //重置托盘图标，避免出现不显示托盘图标的问题
    //判断是否为OSX
    if (process.platform == "darwin") {
      appTray.setPressedImage(aSeleIcon)
    }
    mainWinStatus = "show"
  })
  win.on('hide', () => {

    mainWinStatus = "hide"
  })
  //当窗口获得焦点时触发
  win.on('focus', () => {
    mainWinStatus = "show"

    if (timer) clearInterval(timer);
    appTray.setImage(aIcon) //重置托盘图标，避免出现不显示托盘图标的问题
  })

  //是否全屏记录
  win.on('maximize', (e) => {
    mainWinStatus = "maximize"
    setting.winMaximize = true
    store.set('winMaximize', true);
  })
  win.on('unmaximize', (e) => {
    setting.winMaximize = false
    store.set('winMaximize', false);
  })
  //当窗口失去焦点时触发
  win.on('blur', (e) => {
    mainWinStatus = "blur"
  })

  //窗口大小记录
  win.on('resize', (e) => {
    setting.winWidth = win.getSize()[0]
    setting.winHeight = win.getSize()[1]
    store.set('winWidth', setting.winWidth);
    store.set('winHeight', setting.winHeight);
  })
  //窗口移动位置记录
  win.on('move', (e) => {
    setting.winX = win.getPosition()[0]
    setting.winY = win.getPosition()[1]
    store.set('winX', setting.winX);
    store.set('winY', setting.winY);
  })

  win.on('close', (e) => {

    mainWinStatus = "close"
    if (process.platform == "darwin") {
      e.preventDefault();
      win.hide();
      return
    }
    if (setting.minimize) {
      e.preventDefault();
      win.hide();
    }
  })
  // 当 window 被关闭，这个事件会被触发。
  win.on('closed', () => {

    // 取消引用 window 对象，如果你的应用支持多窗口的话，
    // 通常会把多个 window 对象存放在一个数组里面，
    // 与此同时，你应该删除相应的元素。
    // win = null
    // win.hide();
    if (setWin) {
      setWin.close()
      setWin = null
    }
  })
}catch(e){
  	
  }
  //托盘图标和右键菜单
  var trayMenuTemplate = [
    {
      label: '显示窗口',
      click: function () { //打开相应页面
        win.show()
      }
    },
    {
      label: '设置',
      click: function () {
        let modalPath = path.join(__dirname, 'html/setting.html')
        //判断是否为OSX
        if (process.platform == "darwin") {
          modalPath = path.join('file:', __dirname, 'html/setting.html')
        }
        // const modalPath = `file:/${__dirname}/html/setting.html`
        setWin = new BrowserWindow({
          width: 450,
          height: 360,
          icon: path.join(__dirname, 'img/aicon.png'),
          maximizable: false,
          fullscreen: false,
          webPreferences: {
            nodeIntegration: true
          }
        })
        setWin.on('close', function () { setWin = null })
        setWin.loadURL(modalPath)
        setWin.setMenu(null);
        setWin.show()
      }
    },
    {
      label: '退出',
      click: function () {
        setting.minimize = false //退出应用强制设置取消避免最小化到托盘
        app.quit();
        if (win) win = null
        if (nofcWin) nofcWin = null
        if (setWin) setWin = null
      }
    },
  ];
  // if (process.platform == "darwin") trayMenuTemplate = []
  const Tray = electron.Tray;
  const Menu = electron.Menu
  try{
  if (!appTray) appTray = new Tray(ticon);
}catch(e){
  	
  }
  const contextMenu = Menu.buildFromTemplate(trayMenuTemplate);
  if (process.platform != "darwin") appTray.setContextMenu(contextMenu);
  //appTray.setToolTip('莺语客服')
  //双击托盘图标显示主窗口
//  appTray.on('double-click', () => {
//    if (win) {
//      win.show()
//    }
//    else {
//      if (BrowserWindow) createWindow()
//    }
//  })
  //单点击 1.主窗口显示隐藏切换 2.清除闪烁
  appTray.on("click", function () {

    if (process.platform == "darwin") {
      if (win) {
        win.show()
      }
      else {
        if (BrowserWindow) createWindow()
      }
    }
    else {
      if (timer) {
        clearInterval(timer);
        timer = null
        appTray.setImage(aIcon)
        win.show()
      }
    }
  })

  if (process.platform == "darwin") {
    // app.dock.hide()
    //隐藏菜单
    const menu = Menu.buildFromTemplate([])
    Menu.setApplicationMenu(menu)
  }
  shortcut.addCopyKeyEvent(win)
}

// //防止多开
// var shouldQuit = app.makeSingleInstance(function(commandLine, workingDirectory) {
//   // 当另一个实例运行的时候，这里将会被调用，我们需要激活应用的窗口
//   if (win) {
//     win.show();
//   }
//   return true;
// });

// // 这个实例是多余的实例，需要退出
// if (shouldQuit) {
//   app.quit();
//   return;
// }
// Electron 会在初始化后并准备
// 创建浏览器窗口时，调用这个函数。
// 部分 API 在 ready 事件触发后才能使用。
app.on('ready', createWindow)

// 当全部窗口关闭时退出。
app.on('window-all-closed', () => {

  // 在 macOS 上，除非用户用 Cmd + Q 确定地退出，
  // 否则绝大部分应用及其菜单栏会保持激活。
  if (process.platform !== 'darwin') {
    app.quit()
  }
})

app.on('activate', () => {
  // 在这文件，你可以续写应用剩下主进程代码。
  // 也可以拆分成几个文件，然后用 require 导入。
  if (win === null) {
    //if (BrowserWindow) createWindow()
  }
  else {
    // if (process.platform == 'darwin')
    win.show()
  }

})
app.on('before-quit', (e) => {

  // 在这文件，你可以续写应用剩下主进程代码。
  // 也可以拆分成几个文件，然后用 require 导入。
  // if (app) app.show(), app.exit()
  app.exit()
})
app.on('will-quit', (e) => {

  // 在这文件，你可以续写应用剩下主进程代码。
  // 也可以拆分成几个文件，然后用 require 导入。
  // if (app) app.show(), app.exit()
})

app.on('quit', () => {
  // 在这文件，你可以续写应用剩下主进程代码。
  // 也可以拆分成几个文件，然后用 require 导入。

})
const Menu = electron.Menu
const MenuItem = electron.MenuItem

app.on('browser-window-created', function (event, win) {
  win.webContents.on('context-menu', function (e, params) {
    setTimeout(() => {
      const menu = new Menu()
      // if(menu.items.length > 0) return
      const imageData = imageMenu
      if (imageMenu) {
        menu.append(new MenuItem({ label: '在浏览器打开图片', click: () => _copyImage("open", imageData) }))
        menu.append(new MenuItem({ label: '图片存储为...', click: () => _copyImage("save", imageData) }))
        menu.append(new MenuItem({ label: '复制图片', click: () => _copyImage("image", imageData) }))
        menu.append(new MenuItem({ label: '复制图片地址', click: () => _copyImage("text", imageData) }))
      }
      else {
        menu.append(new MenuItem({
          label: '返回',
          enabled: win.webContents.canGoBack(),
          click(menuItem, browserWindow) {
            let win = BrowserWindow.getFocusedWindow();
            let contents = win.webContents
            contents.goBack()
          }
        }))
        menu.append(new MenuItem({
          label: '前进',
          enabled: win.webContents.canGoForward(),
          click(menuItem, browserWindow) {
            let win = BrowserWindow.getFocusedWindow();
            let contents = win.webContents
            contents.goForward()
          }
        }))
        menu.append(new MenuItem({ label: '复制', role: 'copy' }));
        menu.append(new MenuItem({ label: '粘贴', role: 'paste' }));
        menu.append(new MenuItem({ label: '剪切', role: 'cut' }));
        menu.append(new MenuItem({ label: '重新加载', role: 'reload' }))
      }
      if (mode == 'test') menu.append(new MenuItem({ label: '检查代码', role: 'toggledevtools' }))
      imageMenu = null
      menu.popup(win, params.x, params.y)
    }, 100);
  })
})
// 图片写入剪贴板方法
function _copyImage(type, data) {
  if (!data) return
  const clipboard = electron.clipboard
  if (type == "image") {
    const nativeImage = electron.nativeImage
    const image = nativeImage.createFromDataURL(data.image)
    if (data.image) clipboard.writeImage(image)
  }
  if (type == "text" && data.url) {
    clipboard.writeText(data.url)
  }
  if (type == "open") electron.shell.openExternal(data.url)
  if (type == 'save') win.webContents.downloadURL(data.url)

  if (imageMenu) imageMenu = null
  url = null
}

//监听渲染进程传递的消息
const ipc = require('electron').ipcMain
const handel = require('./shortcut')

//设置的处理
ipc.on('setting', (event, arg) => {
  for (var key in arg) {
    setting[key] = arg[key]
    store.set(key, arg[key]);
    if (key == 'autoStart') {
      if (arg[key]) {
        //勾选则设置开机自启
        let regedit = require('regedit'); //引入regedit
        regedit.putValue({
          'HKCU\\Software\\Microsoft\\Windows\\CurrentVersion\\Run': {
            'yingyuchat': {
              // value : "C:\\Program Files (x86)\\yingyuchat\\yingyuchat.exe",
              // value : app.getAppPath() + '\\..\\..\\yingyuchat.exe',
              value: path.join(path.join(__dirname), '../../../yingyuchat.exe'),
              type: 'REG_SZ'
            }
          }
        }, function (err) {
          console.log(err);
        })
      } else {
        //取消开机自启
        let cp = require('child_process');
        cp.exec("REG DELETE HKCU\\Software\\Microsoft\\Windows\\CurrentVersion\\Run /v yingyuchat /f", function (err) {
          console.log(err);
        });
      }
    }
  }
})

//新消息通知
ipc.on('notice', (event, data) => {
  delete data.option.icon
  if (mainWinStatus == "show" && process.platform == "darwin") return
  if (timer) clearInterval(timer);
  //托盘闪烁
  if (setting['notice.twinkle']) {
    if (process.platform != "darwin") {
      timer = setInterval(function () {
        count++;
        if (count % 2 == 0) {
          appTray.setImage(aIcon)
        } else {
          appTray.setImage(bIcon)
        }
      }, 600);
    }
  }
  //气泡通知
  if (setting['notice.bubble']) {
    if (nofcWin) { nofcWin.close(); nofcWin = null }
    //创建隐藏窗口发送桌面通知
    nofcWin = new BrowserWindow({
      width: 400,
      height: 400,
      show: false,
      webPreferences: {
        nodeIntegration: true
      }
    })

    let modalPath = path.join(__dirname, 'html/notification.html')
    //判断是否为OSX
    if (process.platform == "darwin") {
      modalPath = path.join('file:', __dirname, 'html/notification.html')
    }
    nofcWin.loadURL(modalPath)
    nofcWin.webContents.on('did-finish-load', () => {
      nofcWin.webContents.send('notification', data)
    })
  }
  //窗口激活
  if (setting['notice.winShow']) win.show()
  //闪烁任务栏 mac不处理
  if (process.platform != "darwin") win.flashFrame(true)
})

//用户点击通知
ipc.on('notification-click', (event, toUrl) => {
  if (toUrl) {
    win.show()
    console.log(`${config.workUrl}${toUrl}?electron=${config.edition}`);
    // win.loadURL(`${config.workUrl}${toUrl}?electron=${config.edition}`)
  }
  if (nofcWin) { nofcWin.close(); nofcWin = null }
})

//记录账号密码
ipc.on('userList', (event, data) => {
  const { account, password } = data
  let userList = store.get('userList'), newList = [], identical = false
  userList.map(item => {
    if (item.account != account) newList.push(item)
  })
  newList.unshift({ account, password })
  store.set('userList', newList);
  isLoginOut = false
})
//退出，将未读数清除
ipc.on('LoginOut', () => {
  isLoginOut = true
  //判断是否为OSX
  if (process.platform == "darwin") {
    appTray.setTitle('')
  }
})
//调用截图模块
ipc.on('screenshot', (event, data) => {
  _screenshot()
})

//激活快捷键
ipc.on('addKeyEvent', (event, data) => {
  shortcut.addKeyEvent(win, data.shortcutKeys, _screenshot)
})
//清除快捷键
ipc.on('removeKeyEvent', (event, data) => {
  shortcut.removeKeyEvent(win, data.shortcutKeys)
})
//接受未读数
ipc.on('unread', (event, data) => {
  //判断是否为OSX
  if (process.platform == "darwin") {
    if (data == 0) {
      appTray.setTitle('')
    }
    else {
      if(!isLoginOut) appTray.setTitle(` ${data}`)
    }
  }
})
//接受复制图片
ipc.on('imageMenu', (event, data) => {
  if (data) imageMenu = data
})

function _screenshot() {
  if (process.platform === 'darwin') {
    const child_process = require("child_process");
    child_process.exec(`screencapture -i -c`, (error, stdout, stderr) => {
      // console.log("308", error);
      if (!error) {
        //截图完成，在粘贴板中
        win.webContents.paste()
        win.webContents.send('contact', { type: 'screenshot', data: {} })
      }
    });
    return
  }
  const execFile = require('child_process').execFile;
  var screen_window = execFile(__dirname + '/screen/PrintScr.exe')
  screen_window.on('exit', function (code) {
    // 执行成功返回 1，返回 0 没有截图
    if (code) {
      win.webContents.paste()
      win.webContents.send('contact', { type: 'screenshot', data: {} })
    }
  })
}
// 在这文件，你可以续写应用剩下主进程代码。
// 也可以拆分成几个文件，然后用 require 导入。
